{
 DOSamp Extender - The Nullsoft's DOSamp extension utility.
 Copyright (C) 1998-2003  Henrich Fukna <fuky@azet.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
program DosAmpExtenter;

{$M 8192, 0, 131072}

uses Dos, Lib;

const
  Copyright: String = 'DOSamp Extender'; { identity string }
  Version = '0.9.5';                     { version string constant }
  ExeName = 'DOSAMPEX';                  { project name }

  IniDaxExe: PathStr = 'INIDAX.EXE';     { external configurator }
  DosAmpExe: PathStr = 'DOSAMP.EXE';     { external player }

{$I DOSAMPEX.INC }

{********** class track manager **********************************}

type
  PTrackItem = ^TTrackItem;              { pointer to TrackItem }
  TTrackItem = record                    { TrackItem }
    Number: LongInt;                     { TrackItem number }
    Next: PTrackItem;                    { next TrackItem or nil }
  end;

  TTrackList = object                    { TrackList }
    Count: Integer;                      { number of TrackItem }
    Items: PTrackItem;                   { TrackItem(s) }
    constructor Init;                    { Initialize TrackList }
    destructor Done;                     { Destroy TrackList }

    procedure DeleteAll;                 { Delete all TrackItem(s) }
    function Add(aNumber: LongInt): Boolean;  { Add new TrackItem }
    function Find(aNumber: LongInt): Boolean; { Find TrackItem }
    function GetCount: Integer;          { Get TrackItem count }
  end;

{*
 * Initialize track list.
 *}
constructor TTrackList.Init;
begin
  Count:= 0;
  Items:= nil;
end;

{*
 * Destroy track list.
 *}
destructor TTrackList.Done;
begin
  DeleteAll;
  Items:= nil;
end;

{*
 * Delete all items from track list.
 *}
procedure TTrackList.DeleteAll;
var
  p: PTrackItem;
begin
  while Items <> nil do begin
    p:= Items;
    Items:= Items^.Next;
    Dispose(p);
  end;
  Count:= 0;
end;

{*
 * Adds the track item.
 *
 * @param aNumber               track item number to add
 * @return true if addes, false otherwise
 *}
function TTrackList.Add;
var
  p: PTrackItem;
begin
  Add:= False;
  if Items <> nil then
    if Find(aNumber) then Exit;

  New(p);
  p^.Number:= aNumber;
  p^.Next:= Items;
  Items:= p;
  Inc(Count);
  Add:= True;
end;

{*
 * Finds the track item.
 *
 * @param aNumber               track item number to find
 * @return true if found, false otherwise
 *}
function TTrackList.Find;
var
  p: PTrackItem;
begin
  Find:= False;
  if Items <> nil then begin
    p:= Items;
    while p <> nil do begin
      if p^.Number = aNumber then begin
        Find:= True;
        Exit;
      end;
      p:= p^.Next;
    end;
  end;
end;

{*
 * Get number of track items.
 *
 * @return number of track items
 *}
function TTrackList.GetCount;
begin
  GetCount:= Count;
end;

{********** playback manager *************************************}

{*
 * Setup and call external player.
 *}
procedure Playback;
var
  PlsIni: String;                        { PLS playlist file }
  DosAmpOptions: String;                 { external player options }
  DosAmpFile: String;                    { external player file name }
  DaxOptions: String;                    { extension options }

  DaxRepeat,                             { repeat flag }
  DaxShuffle,                            { shuffle flag }
  DaxReverse: Boolean;                   { reverse flag }

  List: TTrackList;                      { track play history }
  Cnt: LongInt;                          { number of tracks }
  Current: Longint;                      { current track playing }

  I: LongInt;
  S: String;

  {*
   * Initializes tracks.
   *}
  procedure TrackInit;
  begin
    Cnt:= GetPrivateProfileInt(PlsIniSection, PlsIniKey_ListNum, 0, PlsIni);
    I:= 0;                               { get track count }
    if DaxShuffle then begin             { randomize, if shuffle enabled }
      Randomize;
      List.Init;
    end;
  end;

  {*
   * Set next track for playing.
   *}
  procedure TrackNext;
  begin
    if DaxShuffle then                   { if shuffle enabled }
      if List.GetCount = Cnt then        { when is full list, stop playback }
        I:= Cnt+1
      else                               { get random next track }
        repeat I:= Random(Cnt)+1; until List.Add(I)
    else
      Inc(I);                            { otherwise increment }

    if DaxReverse then                   { if reverse enabled }
      Current:= Cnt-I+1                  { recompute track number }
    else
      Current:= I;                       { otherwise plain store }
  end;

  {*
   * Done tracks.
   *}
  procedure TrackDone;
  begin
    if DaxShuffle then List.Done;
  end;

begin
                                         { get options for external player }
  GetPrivateProfileString(DaxIniSection, DaxIniKey_DosAmp, '',
    DosAmpOptions, 80, DaxIni);
                                         { get extended options }
  GetPrivateProfileString(DaxIniSection, DaxIniKey_DosAmpEx, '',
    DaxOptions, 80, DaxIni);
                                         { get playlist file }
  GetPrivateProfileString(DaxIniSection, DaxIniKey_PlayList, '',
    PlsIni, 80, DaxIni);

                                         { setup extended options }
  DaxRepeat:= False;
  DaxShuffle:= False;
  DaxReverse:= False;
  for I:= 0 to GetParseCount(DaxOptions, ' ')-1 do begin
    S:= GetParseString(I, DaxOptions, ' ');
    if S = DaxIniOption_Repeat then DaxRepeat:= True
    else if S = DaxIniOption_Shuffle then DaxShuffle:= True
    else if S = DaxIniOption_Reverse then DaxReverse:= True;
  end;

  repeat
    TrackInit;                           { initialize tracks }
    TrackNext;                           { get next track number }
    while I <= Cnt do begin
      Str(Current, S);
      S:= PlsIniKey_ListItem + S;        { get track file }
      GetPrivateProfileString(PlsIniSection, S, '', DosAmpFile, 80, PlsIni);
      SwapVectors;                       { call external player }
      Exec(DosAmpExe, DosAmpFile + ' ' + DosAmpOptions);
      SwapVectors;
      if DosError <> 0 then begin
        Writeln('Could not execute DOSAMP.EXE');
        Halt(254);
      end;
      if DosExitCode = 255 then begin
        Writeln('Error occurence while DOSAMP.EXE executed.');
        Halt(255);
      end;
      TrackNext;
    end;
    TrackDone;
  until not(DaxRepeat);
end;

{********** main module ******************************************}

{*
 * Prints the syntax.
 *}
procedure Usage;
begin
  Writeln('Usage: ', ExeName, ' [-h -L ', DaxIniSwitch_Cfg, ' ', DaxIniSwitch_PlayList, ']');
  Writeln;
  Writeln('Options:');
  Writeln('  -h          display this help screen');
  Writeln('  -L          display software licence');
  Writeln('  ', DaxIniSwitch_Cfg, '        set configuration (require INIDAX.EXE)');
  Writeln('  ', DaxIniSwitch_PlayList, '   edit playlist (require INIDAX.EXE)');
  Writeln;
  Writeln('For playing is required DOSAMP.EXE');
  Halt(1);
end;

var
  I: Byte;
  Config, PlayList: Boolean;
  Args: String;

begin
  Write(Copyright, ' ', Version, ', ');
  Writeln('Copyright (C) 1998-2003 Henrich Fukna');
  LicenceLine(Copyright, ExeName);
  Writeln;

  Config:= False;
  PlayList:= False;

  I:= 1;
  while I <= ParamCount do begin
    Args:= ParamStr(I);
    if Args = '-h' then Usage
    else if Args = '-L' then begin
      Licence;
      Halt(1);
    end else if Args = DaxIniSwitch_Cfg then Config:= True
    else if Args = DaxIniSwitch_PlayList then PlayList:= True
    else begin
      OptArgErrMsg(ExeName, True);
      Halt(2);
    end;
    Inc(I);
  end;

  Args:= '';                             { set full path to the executables }
  for I:= 0 to GetParseCount(ParamStr(0), '\')-2 do begin
    Args:= Concat(Args, GetParseString(I, ParamStr(0), '\'), '\');
  end;
  DosAmpExe:= Concat(Args, DosAmpExe);
  IniDaxExe:= Concat(Args, IniDaxExe);
  DaxIni:= Concat(Args, DaxIni);

  if Config then begin                   { configuration request }
    SwapVectors;
    Exec(IniDaxExe, DaxIniSwitch_Cfg);   { call external configurator }
    SwapVectors;
    if DosError <> 0 then begin
      Writeln('Could not execute ', IniDaxExe);
      Halt(254);
    end;
    Halt;
  end;

  if PlayList then begin                 { edit playlist request }
    SwapVectors;
    Exec(IniDaxExe, DaxIniSwitch_PlayList); { call external configurator }
    SwapVectors;
    if DosError <> 0 then begin
      Writeln('Could not execute ', IniDaxExe);
      Halt(254);
    end;
    Halt;
  end;

  Playback;                              { setup and call external player }
end.